import 'package:flutter/foundation.dart' show kIsWeb;
import 'package:flutter/widgets.dart';
import 'package:flutter_cross_platform/factory.dart';
import 'package:responsive_builder/responsive_builder.dart';

mixin PlatformNavigatorMixin {

  Factory<Navigator> _webNavigator;
  Factory<Navigator> _deviceNavigator;

  Navigator webNavigator(context);
  Navigator deviceNavigator(context);

  bool _initialized = false;

  get platformNavigator {
    assert(_initialized, "initNavigation() has not called. Call it in app builder");
    if (kIsWeb)
      return _webNavigator.singleton;
    else
      return _deviceNavigator.singleton;
  }

  void initNavigation(BuildContext context) {
    _webNavigator = Factory(factory: () => webNavigator(context));
    _deviceNavigator = Factory(factory: () => deviceNavigator(context));

    _webNavigator.initializeSingleton();
    _deviceNavigator.initializeSingleton();
    _initialized = true;
  }
}


mixin ScreenTypeNavigatorMixin {
  
  Navigator watchNavigator(context);
  Navigator mobileNavigator(context);
  Navigator tabletNavigator(context);
  Navigator desktopNavigator(context);

  Widget navigator(BuildContext context) {
    return ScreenTypeLayout.builder(
      mobile: (ctx) => mobileNavigator(ctx),
      desktop: (ctx) => desktopNavigator(ctx),
      tablet: (ctx) => tabletNavigator(ctx),
      watch: (ctx) => watchNavigator(ctx),
    );
  }
}